// Copyright © 2009, Esko Luontola <www.orfjackal.net>
// This software is released under the Apache License 2.0.
// The license text is at http://www.apache.org/licenses/LICENSE-2.0

package addressbook.domain;

import java.util.*;

/**
 * @author Esko Luontola
 */
public class AddressBook {

    private final List<Contact> contacts = new ArrayList<Contact>();

    public int size() {
        return contacts.size();
    }

    public Contact get(int index) {
        return contacts.get(index);
    }

    public void add(Contact contact) {
        contacts.add(contact);
        sort();
    }

    public void update(Contact existing, Contact updated) {
        contacts.set(contacts.indexOf(existing), updated);
        sort();
    }

    private void sort() {
        Collections.sort(contacts, Contact.SORT_BY_NAME);
    }

    public List<Contact> search(String keywords) {
        List<Contact> found = new ArrayList<Contact>();
        for (Contact contact : contacts) {
            if (matchesKeywords(contact, keywords)) {
                found.add(contact);
            }
        }
        return found;
    }

    private boolean matchesKeywords(Contact contact, String keywords) {
        String needle = keywords.toLowerCase();
        String[] haystack = new String[]{
                contact.getName(),
                contact.getAddress(),
                contact.getEmail(),
                contact.getPhone()
        };
        allToLowerCase(haystack);
        return contains(haystack, needle);
    }

    private static void allToLowerCase(String[] array) {
        for (int i = 0; i < array.length; i++) {
            array[i] = array[i].toLowerCase();
        }
    }

    private static boolean contains(String[] haystack, String needle) {
        for (String s : haystack) {
            if (s.contains(needle)) {
                return true;
            }
        }
        return false;
    }
}
