// Copyright © 2009, Esko Luontola <www.orfjackal.net>
// This software is released under the Apache License 2.0.
// The license text is at http://www.apache.org/licenses/LICENSE-2.0

package addressbook.domain;

import java.util.Comparator;

/**
 * @author Esko Luontola
 */
public class Contact {

    public static final Comparator<Contact> SORT_BY_NAME = new Comparator<Contact>() {
        public int compare(Contact o1, Contact o2) {
            return o1.getName().compareToIgnoreCase(o2.getName());
        }
    };

    private final String name;
    private final String address;
    private final String phone;
    private final String email;

    public Contact() {
        this("", "", "", "");
    }

    public Contact(String name, String address, String phone, String email) {
        this.name = name;
        this.address = address;
        this.phone = phone;
        this.email = email;
    }

    public String getName() {
        return name;
    }

    public String getAddress() {
        return address;
    }

    public String getPhone() {
        return phone;
    }

    public String getEmail() {
        return email;
    }

    public Contact withName(String name) {
        return new Contact(name, address, phone, email);
    }

    public Contact withAddress(String address) {
        return new Contact(name, address, phone, email);
    }

    public Contact withPhone(String phone) {
        return new Contact(name, address, phone, email);
    }

    public Contact withEmail(String email) {
        return new Contact(name, address, phone, email);
    }

    public String toString() {
        return String.format("%s[%s | %s | %s | %s]",
                getClass().getSimpleName(),
                name, address, phone, email);
    }
}
