// Copyright © 2009, Esko Luontola <www.orfjackal.net>
// This software is released under the Apache License 2.0.
// The license text is at http://www.apache.org/licenses/LICENSE-2.0

package addressbook.gui;

import addressbook.domain.*;

import java.util.*;

/**
 * @author Esko Luontola
 */
public class AddressBookController {

    private static final Contact NEW_CONTACT_PLACEHOLDER = new Contact();

    private final AddressBook addressBook;
    private AddressBookView view;

    private String searchKeywords = "";
    private List<Contact> contacts = new ArrayList<Contact>();

    public AddressBookController(AddressBook addressBook) {
        this.addressBook = addressBook;
    }

    public void setView(AddressBookView view) {
        this.view = view;
        doSearch();
    }

    public void fireSearchKeywordsEntered(String keywords) {
        this.searchKeywords = keywords;
        doSearch();
    }

    private void doSearch() {
        this.contacts = addressBook.search(searchKeywords);
        addEmptyRow();
        syncView();
    }

    private void addEmptyRow() {
        contacts.add(NEW_CONTACT_PLACEHOLDER);
    }

    private void syncView() {
        view.updateContacts(Collections.unmodifiableList(contacts));
    }

    public void fireNameChanged(Contact contact, String newName) {
        updateContact(contact, contact.withName(newName));
    }

    public void fireAddressChanged(Contact contact, String newAddress) {
        updateContact(contact, contact.withAddress(newAddress));
    }

    public void firePhoneChanged(Contact contact, String newPhone) {
        updateContact(contact, contact.withPhone(newPhone));
    }

    public void fireEmailChanged(Contact contact, String newEmail) {
        updateContact(contact, contact.withEmail(newEmail));
    }

    private void updateContact(Contact existing, Contact updated) {
        if (existing.equals(NEW_CONTACT_PLACEHOLDER)) {
            addNewContact(updated);
        } else {
            updateExistingContact(existing, updated);
        }
        syncView();
    }

    private void addNewContact(Contact contact) {
        addressBook.add(contact);
        contacts.add(contacts.indexOf(NEW_CONTACT_PLACEHOLDER), contact);
    }

    private void updateExistingContact(Contact existing, Contact updated) {
        addressBook.update(existing, updated);
        contacts.set(contacts.indexOf(existing), updated);
    }
}
